
<!doctype html>
<html>
<head>
<title>{{msg|default(service)}}</title>
<link rel="stylesheet" type="text/css" href="{{url_for('static', filename='style.css')}}">
</head>
<body>
  {{nav.top.render(id='top-navbar')}}
  <break>
   <h1>Control: {{ title }}</h1>

<div class=page>
   <h1>Failing Items</h1>
    {{ table|safe }}
    {{ error_str }}
</div>
 {% if items_checked > 0 %}
 <img src="/graphs/{{ service.replace(' ', '%20') }}/{{ finding.replace(' ', '%20') }}.png", alt="{{ finding }}">
 {% endif %}

</body>

</html>