/*body            { font-family: "Lucida Sans Unicode", "Lucida Grande", sans-serif;}
a, h1, h2       { color: #377ba8; }
h1, h2          { margin: 0; }
h1              { border-bottom: 2px solid #eee; }
h2              { font-size: 1.2em; }*/

nav.navbar a {
  text-decoration: none;
  color: blue;
  font-weight: normal;
}

nav.navbar .active {
  font-weight: bold;
}

nav.navbar > ul {
  list-style: none;
}

nav.navbar ul li {
  display: block;
  width: 150px;
  margin-right: 20px;
  float: left;
}

nav.navbar ul > li > a:hover {
  background-color: #eeeecc;
}

nav.navbar ul .subgroup {
  padding-left: 0px;
}

h1 {
  clear: both;
}

table.dataframe, .dataframe th, .dataframe td {
  border: none;
  border-bottom: 1px solid #C8C8C8;
  border-collapse: collapse;
  text-align:left;
  padding: 10px;
  margin-bottom: 40px;
  font-size: 0.9em;
}

.male th {
    background-color: #add8e6;
    color: white;
}

.female th {
    background-color: #77dd77;
    color: white;
}

tr:nth-child(odd)		{ background-color:#eee; }
tr:nth-child(even)	{ background-color:#fff; }

tr:hover            { background-color: #ffff99;}